;JCM-1 "So Many Things"
	; Version: Episode 7
	; Changelog:
		; Uses monitor routines for sound control.
		; Tightened code up significantly, replacing repetitive code with subroutines and lots of BRA/JMPs to eliminate unnecessary RTSs.
		; Added ability to return to prompt by pressing any key.
		; JCM-1 now sings to you. :o)

.target "65C02", 64				; We're running a 65C02 Processor
.include "JCM-1_BIOS_Ep7.inc"	; We need to know where BIOS and Monitor calls are!

*=$0300

;************************************
;Chip Initialization
				JSR 	Sound_ClearRegs		; Clear all sound chips.
				STZ 	Sound_ChipSelect	; Select Snd0
				
ChannelSetup
				JSR		EnableTone			;
				LDA 	#15  				; Amplitude level 100%
				STA 	Sound_Value			;
				INC		Sound_Register		; Reg 8 - Amplitude Control		
				JSR 	Sound_SendCmd
				INC		Sound_Register		; Reg 9 - Amplitude Control (Amplitude level still 100%)
				JSR 	Sound_SendCmd
				INC		Sound_Register		; Reg 10 - Amplitude Control (Amplitude level still 100%)
				JSR 	Sound_SendCmd
				LDA 	Sound_ChipSelect	; Have we done both chips?
				BNE 	PlaySong			; Yep. Play the song.
				INC 	Sound_ChipSelect	; Otherwise, select Snd1 ...
				BRA 	ChannelSetup		; ... and set up channels again.

PlaySong		JSR 	BassC	;CCCCC
				JSR 	Silence_16th_Enable
				JSR 	M_PrintImm
						.byte "There's so many things "
						.byte $00
				JSR 	MiddlePart
				JSR 	TrebleD_Ending
				
				
				JSR 	BassE	;EEEEE
				JSR 	Sixteenth
				JSR 	M_PrintImm
						.byte "that you can do", $0a, $0d
						.byte $00	
				JSR 	MiddlePart
				JSR 	TrebleC_Ending

				
				JSR 	BassF	;FFFFF
				JSR 	Silence_16th_Enable
				JSR 	M_PrintImm
						.byte "when you fire up "
						.byte $00	
				JSR 	MiddlePart
				JSR 	TrebleD_Ending
				
				
				JSR 	BassD	;DDDDD
				JSR 	Sixteenth
				JSR 	M_PrintImm
						.byte "your JCM-1.", $0a, $0d, $0a, $0d
						.byte $00	
				JSR 	MiddlePart
				JSR 	StopDrum
				JSR 	Sixteenth
				JSR 	TrebleA
				JSR 	Eighth


				JSR 	BassF	;FFFFF
				JSR 	Sixteenth
				JSR 	M_PrintImm
						.byte "Nothing they can say "
						.byte $00
				JSR 	MiddlePart
				JSR		TrebleD_Ending

				
				JSR 	BassG	;GGGGG
				JSR 	Sixteenth
				JSR 	M_PrintImm
						.byte "and nothing they can do", $0a, $0d
						.byte $00
				JSR 	MiddlePart
				JSR 	TrebleC_Ending

				
				JSR 	BassC	;CCCCC
				JSR 	Silence_16th_Enable
				JSR 	M_PrintImm
						.byte "can take you away "
						.byte $00
				JSR 	MiddlePart
				JSR 	TrebleD_Ending

				
				JSR 	BassG	;GGGGG
				JSR 	Sixteenth
				JSR 	M_PrintImm
						.byte "from JCM-1!",$0a,$0d,$0a,$0d,$0a,$0d
						.byte $00
				JSR 	MiddlePart
				JSR 	TrebleC_Ending
JMP	PlaySong

/* SUBROUTINES */

; End Program
Sixteenth		; 1/16th Rest
				LDA 	ACIA0_Status		; Get ACIA Status.
				AND 	#$08				; Was RX buffer empty?
				BEQ 	Init16				; Yep. Continue.
				LDA 	ACIA0_Data			; Clear the ACIA0 Buffer
				JSR 	Sound_ClearRegs		; Silence all sound.
				JMP		M_CmdPrompt			; Bomb to prompt.
				
	Init16		LDX 	#0					; Initialize Outer loop
				LDY 	#0					; Initialize Inner Loop
	Wait16		INX							; Increment Outer Loop
				BNE 	Wait16				; Have we gone around 256 times? If no, go again.
				INY							; Did Outer Loop 256 times. Increment Inner Loop
				CPY 	#$47				; $47 Compare to value that dictates tempo of entire tune.
				BNE 	Wait16				; Until we hit the delay value, go wait Inner Loop times again.
				RTS				

Eighth			JSR 	Sixteenth			; Do 2/16.
				BRA 	Sixteenth			

ThreeSixteenths	JSR 	Sixteenth			; Do 3/16.
				JSR 	Sixteenth
				BRA 	Sixteenth
			
SilenceAll		LDA 	#$3F				;00 111 111: Turns off all channels on /CURRENT/ chip only
				BRA 	MixerControl
	
EnableTone		LDA 	#%00111000			;Enables Tone and Disables Noise on current chip.
	
MixerControl	; Finalize Mixer Control Values
				STA 	Sound_Value			;
				LDA 	#7					;Mixer Control - I/O Enable			(R7 octal)
				STA 	Sound_Register
				JMP 	Sound_SendCmd		; Send it out
	
SnareDrum		INC 	Sound_ChipSelect 	; Select Snd1
				LDA 	#$EF				;Tone Period B, fine tune
				STA 	Sound_Value			;
				LDA 	#2					;Tone Generator Control (R0 octal)
				STA 	Sound_Register		;
				JSR 	Sound_SendCmd
				STZ 	Sound_Value			;Tone Period B, Coarse tune
				LDA 	#3					;Tone Generator Control (R1 octal)
				STA 	Sound_Register		;
				JSR 	Sound_SendCmd

				LDA		#$F					;Pick Noise Period
				STA 	Sound_Value			;
				LDA		#6 					;Set Noise Period
				STA 	Sound_Register		;
				JSR 	Sound_SendCmd
				LDA 	#$1C				; 00 011 100	 enable noise channel C
				BRA 	MixerControl

EnableTone_TrebleC_Eighth				
				JSR 	EnableTone
				JSR 	TrebleC
				BRA 	Eighth				

StopDrum		INC 	Sound_ChipSelect 	; Select Snd1
				LDA 	#$3A				; 00 111 010
				BRA 	MixerControl

KickDrum		INC 	Sound_ChipSelect	; Select Snd1
				LDA 	#$BC				; Tone Period B, fine tune
				STA 	Sound_Value			;
				LDA 	#2					; Tone Generator Control (R0 octal)
				STA 	Sound_Register		;
				JSR 	Sound_SendCmd		;
				LDA 	#3					; Tone Period B, Coarse tune
				STA 	Sound_Value			;
				STA 	Sound_Register		; Tone Generator Control (R1 octal)		
				JSR 	Sound_SendCmd
				LDA 	#$1F				; Pick Noise Period
				STA 	Sound_Value			;
				LDA 	#6 					; Set Noise Period
				STA 	Sound_Register		;
				JSR 	Sound_SendCmd
				LDA 	#$1C				; 00 011 100	 enable noise channel C
				JSR		MixerControl
				JSR 	Sixteenth
				JSR 	StopDrum
				BRA		ThreeSixteenths
				
Snare_16_Stop	JSR		SnareDrum
				JSR		Sixteenth
				BRA		StopDrum
		
TrebleA			LDA 	#$8E				;Tone Period A, fine tune
				BRA		TrebleOut

TrebleC			LDA 	#$77				;Tone Period A, fine tune
				BRA		TrebleOut

TrebleD			LDA 	#$6A				;Tone Period A, fine tune
				BRA		TrebleOut

TrebleE			LDA 	#$5F				;Tone Period A, fine tune
				
TrebleOut		STZ 	Sound_ChipSelect	; Select Snd0
				STA 	Sound_Value			;
				STZ		Sound_Register		;Tone Generator Control (R0 octal)
				JSR 	Sound_SendCmd
				STZ 	Sound_Value			;Tone Period A, Coarse tune
				INC		Sound_Register		;Tone Generator Control (R1 octal)
				JMP 	Sound_SendCmd		
																	
BassC			LDA 	#$DE				;Tone Period A, fine tune
				BRA		BassOut
	
BassE			LDA 	#$7B				;Tone Period A, fine tune
				BRA		BassOut
	
BassF			LDA 	#$66				;Tone Period A, fine tune
				BRA		BassOut
	
BassD			LDA 	#$AA				;Tone Period A, fine tune
				BRA		BassOut
	
BassG			LDA 	#$3f				;Tone Period A, fine tune
				
BassOut			STA 	Sound_Value			;
				INC 	Sound_ChipSelect 	; Select Snd1
				STZ		Sound_Register		;
				JSR 	Sound_SendCmd
				LDA 	#1
				STA		Sound_Value			;Tone Period A, Coarse tune
				STA		Sound_Register		;Tone Generator Control (R1 octal)
				JSR 	Sound_SendCmd
				JSR 	KickDrum
				JMP 	Snare_16_Stop

Silence_16th_Enable
				STZ 	Sound_ChipSelect	; Select Snd0
				JSR 	SilenceAll
				JSR 	Sixteenth
				JMP 	EnableTone

TrebleC_Ending
				JSR		SilenceAll
				JSR		StopDrum
				JSR		Sixteenth
				STZ		Sound_ChipSelect
				JSR 	EnableTone
				JSR 	TrebleC
				JMP 	Eighth

TrebleD_Ending
				JSR		StopDrum
				JSR 	Sixteenth
				JSR 	TrebleD
				JMP 	Eighth
				
MiddlePart
									JSR 	TrebleC
									JSR 	Eighth
									JSR 	TrebleE	
	JSR 	KickDrum
	JSR		SnareDrum
									JSR		TrebleC
									JMP		Sixteenth			