;JCM-1 Episode 7 Stereo Effects Demo
	
.target "65C02", 64				; We're running a 65C02 Processor
.include "JCM-1_BIOS_Ep7.inc"	; We need to know where BIOS and Monitor calls are!

; Storage locations for stereo panning toggle of Warble
WarbleToggle = $E0

; Storage locations for Random Number Generator
SEED0 = $F0
SEED1 = $F1
SEED2 = $F2
SEED3 = $F3
TMP   = $F4
MOD   = $FF

*=$0300
				STZ 	Sound_ChipSelect	; Select Chip 0
				STZ 	WarbleToggle
	
				; Set up SoundVIA to clock data to potentiometers			
				LDA 	#%00011000			; We will use Shift Out mode 110 on the 65C22
				STA 	Sound_ACR			;  ... and send that to Aux Ctrl Register
				LDA 	#$FF				; Set all pins to outputs, just in case.
				STA 	Sound_DDRB 			;  ''
	
Hello			;Say Hello.
				JSR 	M_PrintImm	
				.byte $1B,"[2J",$1B,"[H"
				.byte $0a,$0d
				.byte "       JCM-1 Stereo Demonstration",$0a,$0d,$0a,$0d
				.byte "       1) Passing Car",$0a,$0d
				.byte "       2) Ricochet",$0a,$0d
				.byte "       3) Warbler",$0a,$0d
				.byte "       4) Sweep",$0a,$0d
				.byte $0a,$0d
				.byte "       -= Press ESC to Exit =-",$0a,$0d,$0a,$0d
				.byte "Your Choice: "
				.byte $00
GetInput
				; put cursor back into position
				JSR 	ACIA0_GetByte   	; Get a byte.
				CMP 	#$1B				; Was it Escape?
				BEQ 	GoPrompt			; Yup. Bomb immediately to prompt
				CMP 	#"1"				; "1" Pressed.
				BEQ 	GoCar
				CMP 	#"2"				; "2" Pressed.
				BEQ 	GoRicochet
				CMP 	#"3"				; "3" Pressed.
				BEQ 	GoWarble
				CMP 	#"4"				; "4" Pressed.
				BEQ 	GoSweep
				BRA 	GetInput			; Ignore all other input.
	
GoPrompt
				JSR 	Sound_ClearRegs		; Clear the sound chip
				JMP 	M_CmdPrompt			; Return to monitor

				
				;Jump Table
GoCar			JMP 	PassingCar
GoRicochet		JMP		Ricochet
GoWarble		JMP 	Warbler
GoSweep			JMP 	Sweep

Ricochet
				JSR 	M_PrintImm
				.byte 	"\"Ricochet\""
				.byte 	$00
				JSR 	Sound_ClearRegs		; Silence any current noise.
				STZ		Sound_ChipSelect	; Select Chip 0
				INC 	Sound_ChipSelect	; Bump to Chip 1
				
				LDA 	#%00101110			; xx/Nc Nb Na/Tc Tb Ta  - Noise on B, Tone on A
				STA 	Sound_Value			; 
				LDA 	#7					; Reg 7 - Mixer Control & I/O
				STA 	Sound_Register
				JSR 	Sound_SendCmd		; Send it out
				
				LDA     #$F					; 100% Volume
				STA 	Sound_Value			; 
				INC 	Sound_Register		; Reg 8 - Amplitude Control Channel A
				JSR 	Sound_SendCmd		; Send it out
				
											; 100% Volume 
				INC 	Sound_Register		; Reg 9 - Amplitude Control Channel B
				JSR 	Sound_SendCmd		; Send it out
				
				; short high pitch noise
				LDA 	#31					; Lowest Noise Pitch
				STA		Sound_Value			;
				LDA 	#6					; Register 6 - Noise Pitch
				STA 	Sound_Register 		;
				LDY		#32
				JSR 	RestLoop
				
				; turn off noise
				LDA 	#%00111110			; xx/Nc Nb Na/Tc Tb Ta  - Tone on A
				STA 	Sound_Value			; 
				LDA 	#7					; Reg 7 - Mixer Control & I/O
				STA 	Sound_Register
				JSR 	Sound_SendCmd		; Send it out
				
				; Pitch drop with slight stereo pan.
				LDA 	#0					; Tone Coarse
				STA 	Sound_Value			;
				STA 	Sound_Register		; Reg 1 - Tone A Coarse Tune
				JSR 	Sound_SendCmd		; Send it output
				
				LDY 	#$00				; Y = Pot Value to start
				
	RicoLoop1	LDA 	#32					; A = Pot Chip
				LDX		#%00000011			; X = Pot Number
				JSR		SetPot
				STY 	Sound_Value			; Tone Fine Tune
				JSR 	Sound_SendCmd		; Send it out
				JSR 	Rest
				INY
				CPY		#64
				BNE 	RicoLoop1
				
				;Set Envelope to downward taper
				LDA     #$10				; Volume controlled by Envelope
				STA 	Sound_Value			; 
				LDA		#8
				STA 	Sound_Register		; Reg 8 - Amplitude Control Channel A
				JSR 	Sound_SendCmd		; Send it out
				
				LDA     #$FF				; Envelope Fine Tune
				STA 	Sound_Value			;
				LDA		#11					; Reg 11 - Envelope Fine Tune
				STA 	Sound_Register		;
				JSR 	Sound_SendCmd		; Send it out
				
				LDA     #$0F				; Envelope Coarse Tune
				STA 	Sound_Value			;
				INC 	Sound_Register		; Reg 12 - Envelope Coarse Tune
				JSR 	Sound_SendCmd		; Send it out
				
				LDA     #%00001001			; Downward Taper
				STA 	Sound_Value			;
				INC 	Sound_Register		; Reg 13 - Envelope Shape
				JSR 	Sound_SendCmd		; Send it out

				;finish the stereo pan while the envelope fades out.
	RicoLoop2	LDA 	#32					; A = Pot Chip
				LDX		#%00000011			; X = Pot Number
				JSR		SetPot
				INY
				CPY		#255
				BNE 	RicoLoop2
				
				; Wait a whole bunch while ricochet fades out.
				JSR 	Rest255				
				JSR 	Rest255
				JSR 	Rest255
				JSR 	Rest255
				JSR 	Rest255
				LDA 	ACIA0_Data			; To clear out the ACIA Status
				JMP 	Hello
				
Sweep			; swing pitch and stereo back and forth.
				JSR 	M_PrintImm
				.byte 	"\"Sweep\" - Press any key to return to menu ..."
				.byte 	$00
				JSR 	Sound_ClearRegs		; Silence any current noise.
				STZ		Sound_ChipSelect	; Select Chip 0
				
				LDA 	#%00111110			; xx/Nc Nb Na/Tc Tb Ta  - tone on A only
				STA 	Sound_Value			; 
				LDA 	#7					; Reg 7 - Mixer Control & I/O
				STA 	Sound_Register
				JSR 	Sound_SendCmd		; Send it out
				
				LDA     #$F					; 100% Volume
				STA 	Sound_Value			; 
				INC 	Sound_Register		; Reg 8 - Amplitude Control Channel A
				JSR 	Sound_SendCmd		; Send it out
				
				LDA 	#0					; Tone Coarse
				STA 	Sound_Value			;
				STA 	Sound_Register		; Reg 1 - Tone A Coarse Tune
				JSR 	Sound_SendCmd		; Send it output
				
				LDY 	#$FF				; Y = Pot Value to start
				
	SweepLoop1	LDA 	#16					; A = Pot Chip
				LDX		#%00000011			; X = Pot Number
				JSR		SetPot
				STY 	Sound_Value			; Tone Fine Tune
				JSR 	Sound_SendCmd		; Send it out
				JSR 	Rest
				JSR 	Rest
				JSR 	Rest
				JSR 	Rest
				LDA 	ACIA0_Status		; Get ACIA Status.
				AND 	#$08				; Was RX buffer empty?
				BNE 	ExitSweep			; Nope, go end.
				DEY
				BNE 	SweepLoop1
	
	SweepLoop2	LDA 	#16					; A = Pot Chip
				LDX		#%00000011			; X = Pot Number
				JSR		SetPot
				STY 	Sound_Value			; Tone Fine Tune
				JSR 	Sound_SendCmd		; Send it out
				JSR 	Rest
				JSR 	Rest
				JSR 	Rest
				JSR 	Rest
				LDA 	ACIA0_Status		; Get ACIA Status.
				AND 	#$08				; Was RX buffer empty?
				BNE 	ExitSweep			; Nope, go end.
				INY
				CPY 	#$FF
				BNE 	SweepLoop2
				BRA 	SweepLoop1
	ExitSweep	JMP		ExitWarble
	
Warbler			JSR 	M_PrintImm
				.byte "\"Warbler\" - Press any key to return to menu ..."
				.byte $00
				JSR 	Sound_ClearRegs		; Silence any current noise.
				LDA 	#1
				STA		Sound_ChipSelect	; Select Chip 1
				
				LDA 	#%00111110			; xx/Nc Nb Na/Tc Tb Ta  - tone on A only
				STA 	Sound_Value			; 
				LDA 	#7					; Reg 7 - Mixer Control & I/O
				STA 	Sound_Register
				JSR 	Sound_SendCmd		; Send it out
			
	Warble1		LDA 	ACIA0_Status		; Get ACIA Status.
				AND 	#$08				; Was RX buffer empty?
				BNE 	ExitWarble			; Nope, go end.
				
				LDA     #$F					; Full Volume!
				STA 	Sound_Value			;
				LDA 	#8					; Reg 8 - Amplitude Control Channel A
				STA 	Sound_Register		;
				JSR 	Sound_SendCmd		; Send it out
	
				LDA 	#$FF				; Range of values for random number
				JSR 	RANDOM8				; Tone Fine Tune (random)
				STA 	Sound_Value			;
				STZ		Sound_Register		; Reg 0 - Tone A Fine Tune
				JSR 	Sound_SendCmd		; Send it out
				
				LDA 	#3					; Range of values for random number
				JSR 	RANDOM8				; Tone Coarse (random)
				STA 	Sound_Value			;
				INC 	Sound_Register		; Reg 1 - Tone A Coarse Tune
				JSR 	Sound_SendCmd		; Send it out
						
				LDA 	#32					; A = Pot Chip
				LDX		#%00000011			; X = Pot Number
				LDY 	WarbleToggle		; Y = Pot Value
				JSR		SetPot
				
				LDY 	#64
				JSR 	RestLoop
				
				LDY 	WarbleToggle
				BEQ 	Set255
				STZ 	WarbleToggle
				JMP 	Warble1
	Set255		DEC		WarbleToggle
				JMP 	Warble1
	ExitWarble	LDA 	ACIA0_Data			; To clear out the ACIA Status
				JSR 	Sound_ClearRegs		; Silence any current noise.
				JMP 	Hello
				
PassingCar		JSR 	M_PrintImm
				.byte "\"Passing Car\""
				.byte $00

				JSR 	Sound_ClearRegs		; Silence any current noise.
				LDA 	#1
				STA		Sound_ChipSelect	; Select Chip 1
			
				STZ		Sound_Value			; Noise Period
				LDA		#6					; Reg 6 - Noise Generator Control
				STA 	Sound_Register		;
				JSR 	Sound_SendCmd		; Send it out
				
				LDA 	#%00000110			; xx/Nc Nb Na/Tc Tb Ta  - Noise on all, tone on A
				STA 	Sound_Value			; 
				INC 	Sound_Register 		; Reg 7 - Mixer Control & I/O
				JSR 	Sound_SendCmd		; Send it out
				
				LDA     #%00010000			; --- 1 0000	Amplitude under control of envelope generator
				STA 	Sound_Value
				INC 	Sound_Register		; Reg 8 - Amplitude Control Channel A
				JSR 	Sound_SendCmd		; Send it out
				
				STZ		Sound_Value			; Tone Fine Tune
				STZ		Sound_Register		; Reg 0 - Tone A Fine Tune
				JSR 	Sound_SendCmd		; Send it out
				
				INC 	Sound_Value			; Tone Coarse
				INC 	Sound_Register		; Reg 1 - Tone A Coarse Tune
				JSR 	Sound_SendCmd		; Send it outputs
				
				LDA     #$FF				; Envelope Fine Tune
				STA 	Sound_Value			;
				LDA		#11					; Reg 11 - Envelope Fine Tune
				STA 	Sound_Register		;
				JSR 	Sound_SendCmd		; Send it out
				
				LDA     #$1F				; Envelope Coarse Tune
				STA 	Sound_Value			;
				INC 	Sound_Register		; Reg 12 - Envelope Coarse Tune
				JSR 	Sound_SendCmd		; Send it out
				
				LDA     #%00001101			; Upward Taper
				STA 	Sound_Value			;
				INC 	Sound_Register		; Reg 13 - Envelope Shape
				JSR 	Sound_SendCmd		; Send it out

				LDA 	#32					; A = Pot Chip
				LDX		#%00000011			; X = Pot Number
				LDY		#0					; Y = Pot Value
				JSR		SetPot

			; Wait a whole bunch while car flies in from left.
				JSR 	Rest255
				JSR 	Rest255
				JSR 	Rest255
				JSR 	Rest255
				JSR 	Rest255
				JSR 	Rest255

				STZ		Sound_Value			; It's the sound register twice, and our index loop too. We win three times.

	PitchCross	; The Car Races By!!
				LDA		#6					; Reg 6 - Noise Generator Control
				STA 	Sound_Register		;
				JSR 	Sound_SendCmd		; Send it out

				STZ		Sound_Register		; Reg 0 - Tone A Fine Tune
				JSR 	Sound_SendCmd		; Send it out
				
				LDA 	Sound_Value
				ASL							; x2 ...
				ASL							; x4 ...
				ASL							; Multiply by 8 to derive a Pot value from the Noise Pitch that better matches the Pot's scale of 0-257
				TAY							; Y = Pot Value
				LDA 	#32					; A = Pot Chip
				LDX		#%00000011			; X = Pot Number		
				JSR		SetPot
				
				LDY		#16
				JSR 	RestLoop			; Wait a bit between pitch drops.
				
				INC		Sound_Value			;
				LDA 	Sound_Value
				CMP 	#32					; Have we done it 31 times?
				BNE 	PitchCross			; Nope, go do it again.

				; The car has passed and fades off to the right.
				LDA 	#32					; A = Pot Chip
				LDX		#%00000011			; X = Pot Number
				LDY		#255				; Y = Pot Value
				JSR		SetPot
				LDA     #%00001001			; Downward Taper
				STA 	Sound_Value			;
				LDA		#13					; Reg 13 - Envelope Shape
				STA 	Sound_Register		;
				JSR 	Sound_SendCmd		; Send it out
				
				; Wait a whole bunch while car fades out.
				JSR 	Rest255
				JSR 	Rest255
				JSR 	Rest255
				JSR 	Rest255
				JSR 	Rest255
				JSR 	Rest255
				JSR		Sound_ClearRegs
				LDA 	ACIA0_Data			; To clear out the ACIA Status
				JMP 	Hello				; Go back to input.

/* Subroutines */

Rest			LDX 	#255
	Dexer		DEX
				BNE 	Dexer
				RTS
				
Rest255			LDY 	#$FF				; Do Rest 255 times.
	RestLoop	JSR 	Rest				; Rest Y number of times
				DEY							; Take one down...
				BNE 	RestLoop			; If we're not drunk yet, go grab another beer.
				RTS
				
Sound_IRQLoop	; Checks if Sound_VIA done shifting bits.
				LDA 	Sound_IFR		
				BIT 	#%00000100			; Is the Shift Register Interupt Flag Set?
				BEQ 	Sound_IRQLoop		; Nope! Check again.
				RTS							; Yep! Shift done.

SetPot			; Sets a pot based on the values you pass in the registers.
				STA		Sound_OutRegB		; Set the pot chip.
				STX		Sound_SR			; Select Pot 0
				JSR 	Sound_IRQLoop		; Wait while it sends.
				STY		Sound_SR			; So set it.
				JSR 	Sound_IRQLoop		; Wait while it sends.
				STZ		Sound_OutRegB		; Release the pot chip.
				RTS

; random number generator
; http://www.6502.org/source/integers/random/random.html				
RAND    		LDA #1              ; store 1 in TMP
				LDX #3
RAND1   		STA TMP,X
				LSR
				DEX
				BPL RAND1
				LDY #$20            ; calculate SEED = SEED * RAND4 + TMP
				BNE RAND5           ; branch always
RAND2   		BCC RAND4           ; branch if a zero was shifted out
				CLC                 ; add multiplier to product
				LDX #3
RAND3   		LDA TMP,X
				ADC RAND4,X
				STA TMP,X
				DEX
				BPL RAND3
RAND4   		ROR TMP             ; shift result right
				ROR TMP+1
				ROR TMP+2
				ROR TMP+3
RAND5   		ROR SEED3           ; shift out old seed, and shift in new seed
				ROR SEED2
				ROR SEED1
				ROR SEED0
				DEY
				BPL RAND2
				RTS
RAND6    		.byte  $00,$19,$66,$0D ; multiplier (high byte first!)


RANDOM8  STA MOD    ; store modulus in MOD
         JSR RAND   ; get next seed
         LDA #0     ; multiply SEED by MOD
         STA TMP+2
         STA TMP+1
         STA TMP
         SEC
         ROR MOD    ; shift out modulus, shifting in a 1 (will loop 8 times)
R8A      BCC R8B    ; branch if a zero was shifted out
         CLC        ; add SEED, keep upper 8 bits of product in accumulator
         TAX
         LDA TMP
         ADC SEED0
         STA TMP
         LDA TMP+1
         ADC SEED1
         STA TMP+1
         LDA TMP+2
         ADC SEED2
         STA TMP+2
         TXA
         ADC SEED3
R8B      ROR        ; shift product right
         ROR TMP+2
         ROR TMP+1
         ROR TMP
         LSR MOD    ; loop until all 8 bits of MOD have been shifted out
         BNE R8A
         RTS
				